package com.javaprogrammingforums.domyhomework;

import java.io.*;

import java.util.regex.*;

public class Birthday implements Serializable
{
  public final static long serialVersionUID = 42l;
  public final static Pattern PAT_BIRTHDAY = Pattern.compile("^([0-9]{1,2})[a-z]{0,} ([JFMASOND][a-z]{2,8})$");
  public enum Month
   {
    January(31), February(29), March(31), April(30), May(31), June(30), July(31), August(31), September(30), October(31), November(30), December(31);
    private final int MAX_DAYS;
    Month(int maxDays)
    { MAX_DAYS = maxDays; }
    public void validate(int day) throws IllegalArgumentException
    {
      if (day < 1 || day > MAX_DAYS)
        throw new IllegalArgumentException("Not a valid day in " + this + ": " + day);
    }
   };
  private final int day;
  private final Month month;
  public Birthday(int dayOfMonth, Month m) throws IllegalArgumentException
  {
    m.validate(dayOfMonth);
    day = dayOfMonth;
    month = m;
  }
  public static Birthday parse(String s) throws IllegalArgumentException, NullPointerException
  {
    if (s == null)
      throw new NullPointerException();
    Matcher mat = PAT_BIRTHDAY.matcher(s);
    if (!mat.matches())
      throw new IllegalArgumentException("Bad format for birthday: '" + s + "'");
    Month m;
		try
    {
      m = Enum.valueOf(Month.class, mat.group(2));
    }
    catch (Exception e)
    {
      throw new IllegalArgumentException("Bad month: '" + mat.group(2) + "'", e);
    }
		return new Birthday(Integer.parseInt(mat.group(1)), m);
  }
  public String toString()
  {
    return month + " " + day;
  }
}
